#![feature(const_fn, core_intrinsics, lang_items, libc, start)]
#![cfg_attr(not(all(windows, target_arch = "x86_64")), feature(asm))]
#![no_std]
#![deny(warnings)]

extern crate libc;


// no OS; we must do a busy loop
#[cfg(not(any(unix,
              all(windows, target_arch = "x86_64"))))]
fn forever() -> ! {
    loop {
        #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
        unsafe {
            asm!("pause" :::: "volatile");
        }
    }
}


#[cfg(all(windows, target_arch = "x86_64"))]
fn forever() -> ! {
    #[allow(non_snake_case)]
    extern "system" {
        pub fn Sleep(millis: u32) -> u8;
    }

    loop {
        unsafe {
            Sleep(u32::max_value());
        }
    }
}


// POSIX OS; we should use pause syscall
#[cfg(unix)]
fn forever() -> ! {
    loop {
        #[cfg(target_arch = "x86")]
        unsafe {
            asm!("mov eax, 34" ::: "eax" : "volatile", "intel");
            asm!("int 0x80" :::: "volatile", "intel");
        }

        #[cfg(target_arch = "x86_64")]
        unsafe {
            asm!("mov rax, 34" ::: "rax" : "volatile", "intel");
            asm!("syscall" :::: "volatile", "intel");
        }
    }
}


#[start]
fn start(_argc: isize, _argv: *const *const u8) -> isize {
    forever()
}

#[lang = "eh_personality"]
#[no_mangle]
pub extern "C" fn rust_eh_personality() {}

#[lang = "eh_unwind_resume"]
#[no_mangle]
pub extern "C" fn rust_eh_unwind_resume() {}

#[lang = "panic_fmt"]
#[no_mangle]
pub extern "C" fn rust_begin_panic(_msg: ::core::fmt::Arguments, _file: &'static str, _line: u32) {
    unsafe { ::core::intrinsics::abort() }
}

#[allow(non_snake_case)]
#[no_mangle]
pub extern "C" fn _Unwind_Resume() -> ! {
    forever()
}

#[cfg(windows)]
#[no_mangle]
pub extern "C" fn rust_eh_register_frames(_eh_frame_begin: *const u8, _object: *mut u8) {
    unsafe { ::core::intrinsics::abort() }
}

#[cfg(windows)]
#[no_mangle]
pub extern "C" fn rust_eh_unregister_frames(_eh_frame_begin: *const u8, _object: *mut u8) {
    unsafe { ::core::intrinsics::abort() }
}
